#include "windef.h"
#include "define.h"

class D2IObject {

public:

	virtual UI32 getObjectType() const=0;
	virtual UI8 getObjectVersion() const=0;
	virtual UI8 getDataType() const=0;
	virtual UI16 getEncoding() const=0;
	virtual void * getInterface(UI32)=0;
	virtual CSTR getCSTR()=0;

	virtual ~D2IObject() {}

};

class D2IStreamReader {

public:

	virtual UI8 readStreamByte()=0;
	virtual I32 readStream(BUF,I32)=0;

	virtual ~D2IStreamReader() {}

};

class D2IStreamWriter {

public:

	virtual bool writeStreamByte(UI8)=0;
	virtual I32 writeStream(const BUF,I32)=0;

	virtual ~D2IStreamWriter() {};

};

class D2IStream:public D2IStreamReader,public D2IStreamWriter {

public:

	~D2IStream() {}

};

class D2IBufferInterface {

public:

	virtual I32 getBufferSize() const=0;

	virtual ~D2IBufferInterface() {}

};

class D2IBufferReader:public D2IBufferInterface {

public:

	virtual BUF copyBuffer() const=0;
	virtual UI8 readBufferByte(I32) const=0;
	virtual I32 readBuffer(I32,BUF,I32) const=0;
	virtual bool equalBuffer(const BUF,I32) const=0;
	virtual bool equalBuffer(const D2IBufferReader *) const=0;

	~D2IBufferReader() {}

};

class D2IBufferWriter:public D2IBufferInterface {

public:

	virtual bool writeBufferByte(I32,UI8)=0;
	virtual I32 writeBuffer(I32,const BUF,I32)=0;
	virtual bool fillBuffer(I32,UI8,I32)=0;

	~D2IBufferWriter() {}

};

class D2IFixedBuffer:public D2IStream,public D2IBufferReader,public D2IBufferWriter {

public:

	virtual ~D2IFixedBuffer() {}

};

class D2IBufferAdministrator:public D2IBufferInterface {

public:

	virtual BUF getBuffer() const=0;
	virtual bool clearBuffer(I32)=0;
	virtual bool resizeBuffer(I32)=0;
	virtual bool appendBufferByte(UI8)=0;
	virtual bool appendBuffer(const BUF,I32)=0;

	virtual ~D2IBufferAdministrator() {}

};

class D2IBuffer:public D2IFixedBuffer,D2IBufferAdministrator {

public:

	virtual ~D2IBuffer() {}

};

class D2CFixedBuffer:public D2IObject,public D2IFixedBuffer {

public:

	D2CFixedBuffer();
	D2CFixedBuffer(I32);
	D2CFixedBuffer(const BUF,I32);
	D2CFixedBuffer(const D2IBufferReader *);
	D2CFixedBuffer(const D2CFixedBuffer&);

	D2CFixedBuffer& operator=(const D2CFixedBuffer&);

	virtual ~D2CFixedBuffer();

	//D2IObject
	virtual UI32 getObjectType() const;
	virtual UI8 getObjectVersion() const;
	virtual UI8 getDataType() const;
	virtual UI16 getEncoding() const;
	virtual void * getInterface(UI32);
	virtual CSTR getCSTR();

	// D2IStreamReader
	virtual UI8 readStreamByte() throw(std::out_of_range);
	virtual I32 readStream(BUF,I32);

	// D2IStreamWriter
	virtual bool writeStreamByte(UI8);
	virtual I32 writeStream(const BUF,I32);

	// D2IBufferInterface
	virtual I32 getBufferSize() const;

	// D2IBufferReader
	virtual BUF copyBuffer() const;
	virtual UI8 readBufferByte(I32) const throw(std::out_of_range);
	virtual I32 readBuffer(I32,BUF,I32) const;
	virtual bool equalBuffer(const BUF,I32) const;
	virtual bool equalBuffer(const D2IBufferReader *) const;

	// D2IBufferWriter
	virtual bool writeBufferByte(I32,UI8);
	virtual I32 writeBuffer(I32,const BUF,I32);
	virtual bool fillBuffer(I32,UI8,I32);

protected:

	virtual bool resetBuffer(I32);
	virtual I32 getStreamIndex() const;
	virtual I32 getStreamIndex(I32);
	virtual bool setStreamIndex(I32);
	virtual bool addStreamIndex(I32);

	virtual bool setBufferSize(I32);

	virtual BUF getBufferPointer() const;
	virtual bool setBufferPointer(const BUF);

private:

	I32 m_i32StreamIndex,m_i32BufferSize;

	BUF m_bfBuffer,m_bfCSTR;

};
